/*
    webalizer - a web server log analysis program

    Copyright (C) 1997-2001  Bradford L. Barrett (brad@mrunix.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version, and provided that the above
    copyright and permission notice is included with all distributed
    copies of this or derived software.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

    This software uses the gd graphics library, which is copyright by
    Quest Protein Database Center, Cold Spring Harbor Labs.  Please
    see the documentation supplied with the library for additional
    information and license terms, or visit www.boutell.com/gd/ for the
    most recent version of the library and supporting documentation.
*/

#ifndef _OUTPUT_INTERNAL_H
#define _OUTPUT_INTERNAL_H

/*********************************************/
/* STANDARD INCLUDES                         */
/*********************************************/

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>                           /* normal stuff             */
#include <ctype.h>
#include <sys/utsname.h>
#include <sys/times.h>

/* ensure getopt */
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

/* ensure sys/types */
#ifndef _SYS_TYPES_H
#include <sys/types.h>
#endif

/* some systems need this */
#ifdef HAVE_MATH_H
#include <math.h>
#endif

/* SunOS 4.x Fix */
#ifndef CLK_TCK
#define CLK_TCK _SC_CLK_TCK
#endif

#include "webalizer.h"                        /* main header              */
#include "lang.h"
#include "hashtab.h"
#include "preserve.h"
#include "linklist.h"
#include "graphs.h"
#include "output.h"

/* internal function prototypes */
extern void    write_html_head(char *, FILE *);            /* head of html page   */
extern void    write_html_head_enh(char *, FILE *,int);    /* head of html page   */
extern void    write_html_tail(FILE *);                    /* tail of html page   */
extern void    write_html_tail_enh(FILE *,int);            /* tail of html page   */
extern FILE*   build_new_result_page(char *,char*,char *,int,int ,FILE *);
extern void    close_result_page(FILE*);
extern void    add_menu_item(char *,char*,char *,FILE*);
extern void    add_menu_item_target(char *,char*,char *,FILE*,char*);
extern void    month_links(FILE *);                              /* Page links          */
extern u_long  load_url_array(  UNODEPTR *);               /* load URL array      */
extern u_long  load_site_array( HNODEPTR *);               /* load Site array     */
extern u_long  load_ref_array(  RNODEPTR *);               /* load Refs array     */
extern u_long  load_agent_array(ANODEPTR *);               /* load Agents array   */
extern u_long  load_srch_array( SNODEPTR *);               /* load srch str array */
extern u_long  load_ident_array(INODEPTR *);               /* load ident array    */
extern int     qs_url_cmph( const void*, const void*);     /* compare by hits     */
extern int     qs_url_cmpk( const void*, const void*);     /* compare by kbytes   */
extern int     qs_url_cmpn( const void*, const void*);     /* compare by entrys   */
extern int     qs_url_cmpx( const void*, const void*);     /* compare by exits    */
extern int     qs_site_cmph(const void*, const void*);     /* compare by hits     */
extern int     qs_site_cmpk(const void*, const void*);     /* compare by kbytes   */
extern int     qs_ref_cmph( const void*, const void*);     /* compare by hits     */
extern int     qs_agnt_cmph(const void*, const void*);     /* compare by hits     */
extern int     qs_srch_cmph(const void*, const void*);     /* compare by hits     */
extern int     qs_ident_cmph(const void*, const void*);    /* compare by hits     */
extern int     qs_ident_cmpk(const void*, const void*);    /* compare by kbytes   */

extern int     all_sites_page(u_long, u_long);             /* output site page    */
extern int     all_urls_page(u_long, u_long);              /* output urls page    */
extern int     all_refs_page(u_long, u_long);              /* output refs page    */
extern int     all_agents_page(u_long, u_long);            /* output agents page  */
extern int     all_search_page(u_long, u_long);            /* output search page  */
extern int     all_users_page(u_long, u_long);             /* output ident page   */
extern void    dump_all_sites();                           /* dump sites tab file */
extern void    dump_all_urls();                            /* dump urls tab file  */
extern void    dump_all_refs();                            /* dump refs tab file  */
extern void    dump_all_agents();                          /* dump agents file    */
extern void    dump_all_users();                           /* dump usernames file */
extern void    dump_all_search();                          /* dump search file    */
extern void    top_sites_table(FILE *,int);                /* top n sites table   */
extern void    top_entry_table(FILE*,int);                 /* top n entry/exits   */
extern void    top_refs_table(FILE*);                      /* top n referrers ""  */
extern void    top_agents_table(FILE*);                    /* top n u-agents  ""  */
extern void    top_ctry_table(FILE*);                      /* top n countries ""  */
extern void    top_search_table(FILE*);                    /* top n search strs   */
extern void    top_users_table(FILE*);                     /* top n ident table   */



/* define some colors for HTML */
#define WHITE          "#FFFFFF"
#define BLACK          "#000000"
#define RED            "#FF0000"
#define ORANGE         "#FF8000"
#define LTBLUE         "#0080FF"
#define BLUE           "#0000FF"
#define GREEN          "#00FF00"
#define DKGREEN        "#008040"
#define GREY           "#C0C0C0"
#define LTGREY         "#E8E8E8"
#define YELLOW         "#FFFF00"
#define PURPLE         "#FF00FF"
#define CYAN           "#00E0FF"
#define GRPCOLOR       "#D0D0E0"


/* sort arrays */
extern UNODEPTR *u_array      ;                /* Sort array for URL's     */
extern HNODEPTR *h_array      ;                /* hostnames (sites)        */
extern RNODEPTR *r_array      ;                /* referrers                */
extern ANODEPTR *a_array      ;                /* user agents              */
extern SNODEPTR *s_array      ;                /* search strings           */
extern INODEPTR *i_array      ;                /* ident strings (username) */
extern u_long   a_ctr         ;                 /* counter for sort array   */

extern FILE     *out_fp;


#endif  /* _OUTPUT_INTERNAL_H */
