/*
    webalizer - a web server log analysis program

    Copyright (C) 1997-2001  Bradford L. Barrett (brad@mrunix.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version, and provided that the above
    copyright and permission notice is included with all distributed
    copies of this or derived software.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

    This software uses the gd graphics library, which is copyright by
    Quest Protein Database Center, Cold Spring Harbor Labs.  Please
    see the documentation supplied with the library for additional
    information and license terms, or visit www.boutell.com/gd/ for the
    most recent version of the library and supporting documentation.
*/

#include "output_internal.h"

/* Calculation of durations */
#define seconds(xfer) (((int)((xfer)/1024.0))%60)
#define minutes(xfer) (((int)(((xfer)/1024.0)/60))%60)
#define hours(xfer)   (((int)((xfer)/1024.0))/3600)
#define hours24(xfer) ((((int)((xfer)/1024.0))/3600)%24)
#define days(xfer)    ((((int)((xfer)/1024.0))/3600)/24)

/* internal function prototypes */
void    month_total_table_stream(FILE*);                   /* monthly total table */
void    daily_total_table_stream(FILE*);                   /* daily total table   */
void    hourly_total_table_stream(FILE*);                  /* hourly total table  */
int     write_main_index_stream();
int     write_month_html_stream();
void    dump_all_urls_stream();                            /* dump urls tab file  */
int     all_urls_page_stream(u_long, u_long);              /* output urls page    */
void    top_urls_table_stream(FILE*,int);                  /* top n URL's table   */
void    top_sites_table_stream(FILE *out_fp,int);          /* top n sites table   */
void    top_refs_table_stream(FILE *out_fp);               /* top n referrers ""  */
void    top_agents_table_stream(FILE *out_fp);             /* top n u-agents  ""  */
void    top_ctry_table_stream(FILE *out_fp);               /* top n countries ""  */
void    stream_stat_table(FILE*,int *,int,int);            /* stream duration table */
double  stream_availability(int *);			   /* Calculate the stream source avaibility */


u_long avg_xfer_visit[31]; /* Average download duration for visits */
u_long avg_xfer_hit[31];   /* Average download duration for hits */

/*********************************************/
/* WRITE_MONTH_HTML - does what it says...   */
/*********************************************/

int write_month_html_stream()
{
   int i;
   char html_fname[256];           /* filename storage areas...       */
   char html_mname[256];           /* filename storage areas...       */
   char html_cname[256];           /* filename storage areas...       */
   char findex    [256];           /* filename storage areas...       */
   char png_fname[52];

   char cur_date[100];
   char htitle[256];

   u_long  tm_empty[31];
   u_long  th_empty[24];

   u_long loc_avg_xfer_visit[31];
   u_long loc_avg_xfer_hit  [31];
   int j;
   int tot_visit=0,tot_hit=0;
   u_long tot_xfer = 0;

   FILE *cout_fp = NULL;
   FILE *ccout_fp = NULL;
   FILE *cccout_fp = NULL;
   FILE *ccccout_fp = NULL;
   FILE *fout_fp = NULL;
   FILE *mout_fp = NULL;

   /* Init empty arrays */
   for( i = 0 ; i<31 ; i++ ) tm_empty[i] = 0;
   for( i = 0 ; i<24 ; i++ ) th_empty[i] = 0;
   
   if (verbose>1)
      printf("%s %s %d\n",msg_gen_rpt, l_month[cur_month-1], cur_year); 

   /* update history */
   i=cur_month-1;
   hist_month[i] =  cur_month;
   hist_year[i]  =  cur_year;
   hist_hit[i]   =  t_hit;
   hist_visit[i] =  t_visit;
   hist_site[i]  =  t_site;
   hist_xfer[i]  =  t_xfer/1024;
   hist_fday[i]  =  f_day;
   hist_lday[i]  =  l_day;
   hist_xfer_stream[i]  =  stream_t_xfer/1024; /* because stream_t_xfer is in seconds*1024 */
   hist_hit_stream[i]   =  stream_t_hit;
   hist_visit_stream[i] =  stream_t_visit;

   /* Calculate the average listening time for visits/hits */
   for( j = 0 ; j<31 ; j++ )
   {
	tot_xfer += stream_tm_xfer[j];
	tot_visit+= stream_tm_visit[j];
	tot_hit  += stream_tm_hit  [j];

	avg_xfer_visit[j] = (stream_tm_visit[j]==0)? 0 : stream_tm_xfer[j]/stream_tm_visit[j];
	avg_xfer_hit  [j] = (stream_tm_hit  [j]==0)? 0 : stream_tm_xfer[j]/stream_tm_hit  [j];
	/* Because stream_tm_xfer is in seconds*1024 */
	loc_avg_xfer_visit[j] = avg_xfer_visit[j]>>10;
	loc_avg_xfer_hit  [j] = avg_xfer_hit  [j]>>10;
   }

   /* average xfer for visit and hit in history */
   hist_avtv[i] = (tot_visit==0) ? 0 : tot_xfer/tot_visit/1024; /* in seconds */
   hist_avth[i] = (tot_hit  ==0) ? 0 : tot_xfer/tot_hit  /1024; /* in seconds */

   /* fill in filenames */
   sprintf(html_fname,"usage_%04d%02d.%s",cur_year,cur_month,html_ext);
   sprintf(findex    ,"index.%s",html_ext);
   sprintf(html_mname,"usage_menu_%04d%02d.%s",cur_year,cur_month,html_ext);
   sprintf(html_cname,"month_total_%04d%02d.%s",cur_year,cur_month,html_ext);

   /* now do html stuff... */

   sprintf(cur_date,"%s %d",l_month[cur_month-1],cur_year);

   /* FRAME SET */
   if ( (fout_fp=open_out_file(html_fname))==NULL ) return 1;
   fprintf(fout_fp,"<html><title>Webalizer: %s</title><frameset rows=\"150,*\" framespacing=\"0\" border=\"0\" frameborder=\"0\"><frame src=\"%s\" marginwidth=\"0\" marginheight=\"0\" scrolling=\"no\"><frame name=\"content\" src=\"%s\"  marginwidth=\"0\" marginheight=\"0\" scrolling=\"yes\"><noframes><body></body></noframes></frameset></html>",
		cur_date,html_mname,html_cname);
   fclose(fout_fp);

   /************ MENU PAGE */
   if ( (mout_fp=open_out_file(html_mname))==NULL ) return 1;
   write_html_head(cur_date, mout_fp);
   add_menu_item_target("Index","Index",findex,mout_fp,"_top"); 

   /************ MONTHLY TOTALS */
   if ((cout_fp=build_new_result_page(msg_mtot_stream,msg_mtot_stream,"month_total",cur_year,cur_month,mout_fp))==NULL) return 1;
   month_total_table_stream(cout_fp);
   close_result_page(cout_fp);

   /************ Duration/Listeners */
   if ((cout_fp=build_new_result_page(msg_list_dur,msg_list_dur,"listeners_duration",cur_year,cur_month,mout_fp))==NULL) return 1;
   sprintf(png_fname,"monthly_duration_listeners_%04d%02d.png",cur_year,cur_month);
   sprintf(htitle,"%s %s %d",msg_list_dur_ttl,l_month[cur_month-1],cur_year);
   fprintf(cout_fp,"<IMG SRC=\"%s\" ALT=\"%s\"><P>\n",png_fname,htitle);
   month_stream_usage(png_fname,htitle,stream_tm_dur_list,MAX_DURATION_GR,1,msg_list_dur_min,"",COLOR1);
     /* Less details */
     if ((ccout_fp=build_new_result_page(msg_list_dur,msg_list_dur_zoom,"listeners_duration_2",cur_year,cur_month,cout_fp))==NULL) return 1;
     sprintf(png_fname,"monthly_duration_listeners_2_%04d%02d.png",cur_year,cur_month); 
     sprintf(htitle,"%s %s %d",msg_list_dur_ttl,l_month[cur_month-1],cur_year);
     fprintf(ccout_fp,"<IMG SRC=\"%s\" ALT=\"%s\"><P>\n",png_fname,htitle);
     month_stream_usage(png_fname,htitle,stream_tm_dur_list,MAX_DURATION_GR,5,msg_list_dur_min,"",COLOR1);
       /* Less details */
       if ((cccout_fp=build_new_result_page(msg_list_dur,msg_list_dur_zoom,"listeners_duration_3",cur_year,cur_month,ccout_fp))==NULL) return 1;
       sprintf(png_fname,"monthly_duration_listeners_3_%04d%02d.png",cur_year,cur_month); 
       sprintf(htitle,"%s %s %d",msg_list_dur_ttl,l_month[cur_month-1],cur_year);
       fprintf(cccout_fp,"<IMG SRC=\"%s\" ALT=\"%s\"><P>\n",png_fname,htitle);
       month_stream_usage(png_fname,htitle,stream_tm_dur_list,MAX_DURATION_GR,10,msg_list_dur_min,"",COLOR1);
         /* Less details */
         if ((ccccout_fp=build_new_result_page(msg_list_dur,msg_list_dur_zoom,"listeners_duration_4",cur_year,cur_month,cccout_fp))==NULL) return 1;
         sprintf(png_fname,"monthly_duration_listeners_4_%04d%02d.png",cur_year,cur_month); 
         sprintf(htitle,"%s %s %d",msg_list_dur_ttl,l_month[cur_month-1],cur_year);
         fprintf(ccccout_fp,"<IMG SRC=\"%s\" ALT=\"%s\"><P>\n",png_fname,htitle);
         month_stream_usage(png_fname,htitle,stream_tm_dur_list,MAX_DURATION_GR,20,msg_list_dur_min,"",COLOR1);
         stream_stat_table(ccccout_fp,stream_tm_dur_list,MAX_DURATION_GR,20);
         close_result_page(ccccout_fp);
       stream_stat_table(cccout_fp,stream_tm_dur_list,MAX_DURATION_GR,10);
       close_result_page(cccout_fp);
     stream_stat_table(ccout_fp,stream_tm_dur_list,MAX_DURATION_GR,5);
     close_result_page(ccout_fp);
   stream_stat_table(cout_fp,stream_tm_dur_list,MAX_DURATION_GR,1);
   close_result_page(cout_fp);

   /*********** Hourly Hits over a month */
   if ((cout_fp=build_new_result_page(msg_hhm_stream,msg_hhms_stream,"month_hourly_hits",cur_year,cur_month,mout_fp))==NULL) return 1;
   sprintf(png_fname,"monthly_hour_hits_%04d%02d.png",cur_year,cur_month);
   sprintf(htitle,"%s (%s %d %s) %s %d",msg_hhm_stream,msg_sample_fq,stream_period,msg_minutes,l_month[cur_month-1],cur_year);
   fprintf(cout_fp,"<IMG SRC=\"%s\" ALT=\"%s\"><P>\n",png_fname,htitle);
   /* write table ?*/
   month_hourly_stream_usage(png_fname,htitle,cur_month,cur_year,stream_tmh_hit,60/stream_period,MONTHLY,msg_h_hits,"",green);

   /* Source activity: source is on */
   sprintf(png_fname,"monthly_hour_source_cxi_%04d%02d.png",cur_year,cur_month);
   sprintf(htitle,"%s (%s %d %s) %s %d",msg_hsam_stream,msg_sample_fq,stream_period,msg_minutes,l_month[cur_month-1],cur_year);
   fprintf(cout_fp,"<IMG SRC=\"%s\" ALT=\"%s\"><P>\n",png_fname,htitle);
   /* write table ?*/
   month_hourly_stream_usage(png_fname,htitle,cur_month,cur_year,stream_src_tmh_hit,60/stream_period,MONTHLY,msg_sa_stream,"",green);
   /* Source activity: Deconnections */
   sprintf(png_fname,"monthly_hour_source_cxo_%04d%02d.png",cur_year,cur_month);
   sprintf(htitle,"%s (%s %d %s) %s %d",msg_hsdm_stream,msg_sample_fq,stream_period,msg_minutes,l_month[cur_month-1],cur_year);
   fprintf(cout_fp,"<IMG SRC=\"%s\" ALT=\"%s\"><P>\n",png_fname,htitle);
   /* write table ?*/
   month_hourly_stream_usage(png_fname,htitle,cur_month,cur_year,stream_src_tmh_cxo,60/stream_period,MONTHLY,msg_sd_stream,"",red);
   close_result_page(cout_fp);

   /*********** Hourly Hits over a week */
   if ((cout_fp=build_new_result_page(msg_hhw_stream,msg_hhws_stream,"week_hourly_hits",cur_year,cur_month,mout_fp))==NULL) return 1;
   sprintf(png_fname,"week_hour_hits_%04d%02d.png",cur_year,cur_month);
   sprintf(htitle,"%s (%s %d %s) %s %d",msg_hhw_stream,msg_sample_fq,stream_period,msg_minutes,l_month[cur_month-1],cur_year);
   fprintf(cout_fp,"<IMG SRC=\"%s\" ALT=\"%s\"><P>\n",png_fname,htitle);
   /* write table ?*/
   close_result_page(cout_fp);
   month_hourly_stream_usage(png_fname,htitle,cur_month,cur_year,stream_tmh_hit,60/stream_period,WEEKLY,msg_h_hits,"",green);

   /*********** Hourly Hits over a day */
   if ((cout_fp=build_new_result_page(msg_hhd_stream,msg_hhds_stream,"day_hourly_hits",cur_year,cur_month,mout_fp))==NULL) return 1;
   sprintf(png_fname,"day_hour_hits_%04d%02d.png",cur_year,cur_month);
   sprintf(htitle,"%s (%s %d %s) %s %d",msg_hhd_stream,msg_sample_fq,stream_period,msg_minutes,l_month[cur_month-1],cur_year);
   fprintf(cout_fp,"<IMG SRC=\"%s\" ALT=\"%s\"><P>\n",png_fname,htitle);
   /* write table ?*/
   close_result_page(cout_fp);
   month_hourly_stream_usage(png_fname,htitle,cur_month,cur_year,stream_tmh_hit,60/stream_period,DAILY,msg_h_hits,"",green);

   /********** Daily stuff */
   if (daily_graph || daily_stats)
   {
      if ((cout_fp=build_new_result_page(msg_hlnk_ds,msg_hlnk_ds,"daily_usage",cur_year,cur_month,mout_fp))==NULL) return 1;
      sprintf(htitle,"%s %s %d",msg_hmth_du_stream,l_month[cur_month-1],cur_year);
      if (daily_graph)
      {
         sprintf(png_fname,"daily_usage_%04d%02d.png",cur_year,cur_month);
         fprintf(cout_fp,"<IMG SRC=\"%s\" ALT=\"%s\" HEIGHT=400 WIDTH=512><P>\n",png_fname,htitle);
         month_graph6_msg(png_fname,htitle,cur_month,cur_year,            		    
                    stream_tm_hit,msg_h_hits,green,tm_empty,"",COLOR2,
                    loc_avg_xfer_hit  ,msg_d_avgth,COLOR5,  
                    stream_tm_xfer,msg_h_xfer_stream,COLOR4,
                    stream_tm_visit,msg_h_visits,COLOR6,    
                    loc_avg_xfer_visit,msg_d_avgtv,COLOR3, 
		    1024
		    );
      }
      if (daily_stats) daily_total_table_stream(cout_fp);
      close_result_page(cout_fp);
   }

   /*********** Hourly stuff */
   if (hourly_graph || hourly_stats)      /* Hourly stuff */
   {
      if ((cout_fp=build_new_result_page(msg_hlnk_hs,msg_hlnk_hs,"hourly_usage",cur_year,cur_month,mout_fp))==NULL) return 1;
      sprintf(htitle,"%s %s %d",msg_hmth_du_stream,l_month[cur_month-1],cur_year);

      if (hourly_graph) 
      {
         u_long local_stream_th_xfer[24];
         int j;
         sprintf(png_fname,"hourly_usage_%04d%02d.png",cur_year,cur_month);
         fprintf(cout_fp,"<IMG SRC=\"%s\" ALT=\"%s\" HEIGHT=256 WIDTH=512><P>\n",png_fname,htitle);
         day_graph3_msg(png_fname,htitle,
                     stream_th_cxi,msg_h_cxi,COLOR5,
                     stream_th_cxo,msg_h_cxo,red,
                     stream_th_hit,msg_h_hits,green);

         for( j = 0 ; j<24 ; j++ ) local_stream_th_xfer[j] = (u_long)stream_th_xfer[j];

         sprintf(png_fname,"hourly_duration_%04d%02d.png",cur_year,cur_month);
         fprintf(cout_fp,"<IMG SRC=\"%s\" ALT=\"%s\" HEIGHT=256 WIDTH=512><P>\n",png_fname,htitle);
         day_graph3_msg(png_fname,htitle,
                     th_empty,"",COLOR1,
                     th_empty,"",COLOR1,
                     local_stream_th_xfer,msg_h_xfer_stream,COLOR4 );
      }
      if (hourly_stats) hourly_total_table_stream(cout_fp);
      close_result_page(cout_fp);
   }

   /* Do URL related stuff here, sorting appropriately                      */
   if ( (a_ctr=load_url_array(NULL)) )
   {
    if ( (u_array=malloc(sizeof(UNODEPTR)*(a_ctr))) !=NULL )
    {
     a_ctr=load_url_array(u_array);        /* load up our sort array        */
     if (ntop_urls || dump_urls)
     {
       qsort(u_array,a_ctr,sizeof(UNODEPTR),qs_url_cmph);
       if (ntop_urls) 
       {
          if ((cout_fp=build_new_result_page(msg_tot_url_h,msg_tot_url_hs,"top_urls_hits",cur_year,cur_month,mout_fp))==NULL) return 1;
          top_urls_table_stream(cout_fp,0);   /* Top URL's (by hits)           */
      	  close_result_page(cout_fp);
       }
       if (dump_urls) dump_all_urls_stream();     /* Dump URLS tab file            */
     }
     if (ntop_urlsK)                       /* Top URL's (by kbytes)         */
     {
      	qsort(u_array,a_ctr,sizeof(UNODEPTR),qs_url_cmpk); 
        if ((cout_fp=build_new_result_page(msg_tot_url_d,msg_tot_url_ds,"top_urls_kb",cur_year,cur_month,mout_fp))==NULL) return 1;
      	top_urls_table_stream(cout_fp,1); 
      	close_result_page(cout_fp);
     }
     free(u_array);
    }
    else if (verbose) fprintf(stderr,"%s [u_array]\n",msg_nomem_tu); /* err */
   }

   /* do hostname (sites) related stuff here, sorting appropriately...      */
   if ( (a_ctr=load_site_array(NULL)) )
   {
    if ( (h_array=malloc(sizeof(HNODEPTR)*(a_ctr))) !=NULL )
    {
     a_ctr=load_site_array(h_array);       /* load up our sort array        */
     if (ntop_sites || dump_sites)
     {
       qsort(h_array,a_ctr,sizeof(HNODEPTR),qs_site_cmph);
       if (ntop_sites) 
       {
          if ((cout_fp=build_new_result_page(msg_tot_sites_h,msg_tot_sites_hs,"top_sites",cur_year,cur_month,mout_fp))==NULL) return 1;
          top_sites_table_stream(cout_fp,0); /* Top sites table (by hits)     */
      	  close_result_page(cout_fp);
       }
       if (dump_sites) dump_all_sites();   /* Dump sites tab file           */
     }
     if (ntop_sitesK)                      /* Top Sites table (by kbytes)   */
     {
       qsort(h_array,a_ctr,sizeof(HNODEPTR),qs_site_cmpk);
       if ((cout_fp=build_new_result_page(msg_tot_sites_d,msg_tot_sites_ds,"top_sites_kb",cur_year,cur_month,mout_fp))==NULL) return 1;
       top_sites_table_stream(cout_fp,1);
       close_result_page(cout_fp);
     }
     free(h_array);
    }
    else if (verbose) fprintf(stderr,"%s [h_array]\n",msg_nomem_ts); /* err */
   }

   /* do referrer related stuff here, sorting appropriately...              */
   if ( (a_ctr=load_ref_array(NULL)) )
   {
    if ( (r_array=malloc(sizeof(RNODEPTR)*(a_ctr))) != NULL)
    {
     a_ctr=load_ref_array(r_array);
     if (ntop_refs || dump_refs)
     {
       qsort(r_array,a_ctr,sizeof(RNODEPTR),qs_ref_cmph);
       if (ntop_refs) 
       {	  
          if ((cout_fp=build_new_result_page(msg_tot_ref,msg_tot_ref,"top_referers",cur_year,cur_month,mout_fp))==NULL) return 1;
       	  top_refs_table(cout_fp);   /* Top referrers table            */
      	  close_result_page(cout_fp);
       }
       if (dump_refs) dump_all_refs();    /* Dump referrers tab file        */
     }
     free(r_array);
    }
    else if (verbose) fprintf(stderr,"%s [r_array]\n",msg_nomem_tr); /* err */
   }

   /* do search string related stuff, sorting appropriately...              */
   if ( (a_ctr=load_srch_array(NULL)) )
   {
    if ( (s_array=malloc(sizeof(SNODEPTR)*(a_ctr))) != NULL)
    {
     a_ctr=load_srch_array(s_array);
     if (ntop_search || dump_search)
     {
       qsort(s_array,a_ctr,sizeof(SNODEPTR),qs_srch_cmph);
       if (ntop_search) 
       {
          if ((cout_fp=build_new_result_page(msg_tot_ss,msg_tot_sss,"top_searchs",cur_year,cur_month,mout_fp))==NULL) return 1;
       	  top_search_table(cout_fp); /* top search strings table     */
      	  close_result_page(cout_fp);
       }
       if (dump_search) dump_all_search();  /* dump search string tab file  */
     }
     free(s_array);
    }
    else if (verbose) fprintf(stderr,"%s [s_array]\n",msg_nomem_tsr);/* err */
   }

   /* do ident (username) related stuff here, sorting appropriately...      */
   if ( (a_ctr=load_ident_array(NULL)) )
   {
    if ( (i_array=malloc(sizeof(INODEPTR)*(a_ctr))) != NULL)
    {
     a_ctr=load_ident_array(i_array);
     if (ntop_users || dump_users)
     {
       qsort(i_array,a_ctr,sizeof(INODEPTR),qs_ident_cmph);
       if (ntop_users) 
       {
          if ((cout_fp=build_new_result_page(msg_tot_users,msg_tot_users,"top_users",cur_year,cur_month,mout_fp))==NULL) return 1;
       	  top_users_table(cout_fp); /* top usernames table      */
      	  close_result_page(cout_fp);
       }
       if (dump_users) dump_all_users();  /* dump usernames tab file        */
     }
     free(i_array);
    }
    else if (verbose) fprintf(stderr,"%s [i_array]\n",msg_nomem_ti); /* err */
   }

   /* do user agent related stuff here, sorting appropriately...            */
   if ( (a_ctr=load_agent_array(NULL)) )
   {
    if ( (a_array=malloc(sizeof(ANODEPTR)*(a_ctr))) != NULL)
    {
     a_ctr=load_agent_array(a_array);
     if (ntop_agents || dump_agents)
     {
       qsort(a_array,a_ctr,sizeof(ANODEPTR),qs_agnt_cmph);
       if (ntop_agents) 
       {
          if ((cout_fp=build_new_result_page(msg_tot_ua,msg_tot_ua,"top_ua",cur_year,cur_month,mout_fp))==NULL) return 1;
          top_agents_table(cout_fp); /* top user agents table        */
      	  close_result_page(cout_fp);
       }
       if (dump_agents) dump_all_agents();  /* dump user agents tab file    */
     }
     free(a_array);
    }
    else if (verbose) fprintf(stderr,"%s [a_array]\n",msg_nomem_ta); /* err */
   }

   if (ntop_ctrys ) 
   {
      if ((cout_fp=build_new_result_page(msg_tot_country,msg_tot_country,"top_countries",cur_year,cur_month,mout_fp))==NULL) return 1;
      top_ctry_table_stream(cout_fp);     /* top countries table            */
      close_result_page(cout_fp);
   }

   write_html_tail_enh(mout_fp,0); /* finish up the HTML document    */
   fclose(mout_fp);
   return (0);
}

/*********************************************/
/* Calculate the source availability in percents */
/*********************************************/

double stream_availability(int *src_hits)
{
   int i,days_in_month=days_in_month=(l_day-f_day)+1;
   int nb_div = days_in_month*24*60/stream_period;
   double total = 0;

   /* 1 hit per period=good, 0=bad, more than 1=bad */
   /* If more than one hit, then evaluate the availability compared to the stream period */
   /* period=30 min, 10 hits, then, avalability is 60% for this period */
   for( i = 0 ; i<nb_div ; i++ )
   {
     total += (src_hits[i]==0) ? 0.0l : (src_hits[i]==1) ? 1.0l : (src_hits[i]<stream_period) ? (stream_period-src_hits[i])/stream_period : 0;
   }

   return (total/(double)nb_div)*100.0;
}

/*********************************************/
/* MONTH_TOTAL_TABLE - monthly totals table  */
/*********************************************/

void month_total_table_stream(FILE*out_fp)
{
   int i,days_in_month;
   u_long max_hits=0,max_visits=0;
   double max_xfer=0.0;
   double source_avl = stream_availability(stream_src_tmh_hit);/* Calculate the stream source avaibility */

   days_in_month=(l_day-f_day)+1;
   for (i=0;i<31;i++)
   {  /* Get max/day values */
      if (stream_tm_hit[i]>max_hits)     max_hits  = stream_tm_hit[i];
      if (stream_tm_visit[i]>max_visits) max_visits= stream_tm_visit[i];
      if (stream_tm_xfer[i]>max_xfer)    max_xfer  = stream_tm_xfer[i];
   }

   fprintf(out_fp,"<TABLE WIDTH=510 BORDER=2 CELLSPACING=1 CELLPADDING=1>\n");
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   fprintf(out_fp,"<TR><TH COLSPAN=3 ALIGN=center BGCOLOR=\"%s\">"           \
      "%s %s %d</TH></TR>\n",GREY,msg_mtot_ms,l_month[cur_month-1],cur_year);
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   /* Total Hits */
   fprintf(out_fp,"<TR><TD WIDTH=380><FONT SIZE=\"-1\">%s</FONT></TD>\n"     \
      "<TD ALIGN=right COLSPAN=2><FONT SIZE=\"-1\"><B>%lu</B>"               \
      "</FONT></TD></TR>\n",msg_mtot_th_stream,stream_t_hit);
   /* Total Visits */
   fprintf(out_fp,"<TR><TD WIDTH=380><FONT SIZE=\"-1\">%s %s</FONT></TD>\n"  \
      "<TD ALIGN=right COLSPAN=2><FONT SIZE=\"-1\"><B>%lu</B>"               \
      "</FONT></TD></TR>\n",msg_h_total, msg_h_visits, stream_t_visit);
   /* Total XFer */
   fprintf(out_fp,"<TR><TD WIDTH=380><FONT SIZE=\"-1\">%s</FONT></TD>\n"     \
      "<TD ALIGN=right COLSPAN=2><FONT SIZE=\"-1\"><B>%d %s %02d:%02d (%d:%02d:%02d)</B>"      \
      "</FONT></TD></TR>\n",msg_mtot_tx_stream,days(stream_t_xfer),msg_days_ext,hours24(stream_t_xfer),minutes(stream_t_xfer),hours(stream_t_xfer),minutes(stream_t_xfer),seconds(stream_t_xfer));
   /* Total XFer in MB for the listeners */
   fprintf(out_fp,"<TR><TD WIDTH=380><FONT SIZE=\"-1\">%s</FONT></TD>\n"     \
      "<TD ALIGN=right COLSPAN=2><FONT SIZE=\"-1\"><B>%.03f %s</B>"           \
      "</FONT></TD></TR>\n",msg_list_xfer, stream_t_xfer_kb/(1024*1024),msg_mbytes);
   /* Total time for source */
   fprintf(out_fp,"<TR><TD WIDTH=380><FONT SIZE=\"-1\">%s</FONT></TD>\n"     \
      "<TD ALIGN=right COLSPAN=2><FONT SIZE=\"-1\"><B>%d %s %02d:%02d (%d:%02d:%02d)</B>"      \
      "</FONT></TD></TR>\n",msg_mtot_tx_src_stream,days(stream_src_t_xfer),msg_days_ext,hours24(stream_src_t_xfer),minutes(stream_src_t_xfer),hours(stream_src_t_xfer),minutes(stream_src_t_xfer),seconds(stream_src_t_xfer));
   /* Total XFer in MB for the source */
   fprintf(out_fp,"<TR><TD WIDTH=380><FONT SIZE=\"-1\">%s</FONT></TD>\n"     \
      "<TD ALIGN=right COLSPAN=2><FONT SIZE=\"-1\"><B>%.03f %s</B>"           \
      "</FONT></TD></TR>\n",msg_src_xfer, stream_src_t_xfer_kb/(1024*1024),msg_mbytes);
   /* Source avaibility */
   fprintf(out_fp,"<TR><TD WIDTH=380><FONT SIZE=\"-1\">%s</FONT></TD>\n"     \
      "<TD ALIGN=right COLSPAN=2><FONT SIZE=\"-1\"><B>%02.2f %%</B>"           \
      "</FONT></TD></TR>\n",msg_src_avl, source_avl);
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   /**********************************************/
   /* Unique Sites */
   fprintf(out_fp,"<TR>"                                                     \
      "<TD WIDTH=380><FONT SIZE=\"-1\">%s</FONT></TD>\n"                     \
      "<TD ALIGN=right COLSPAN=2><FONT SIZE=\"-1\"><B>%lu</B>"               \
      "</FONT></TD></TR>\n",msg_mtot_us,t_site);
   /* Unique URL's */
   fprintf(out_fp,"<TR>"                                                     \
      "<TD WIDTH=380><FONT SIZE=\"-1\">%s</FONT></TD>\n"                     \
      "<TD ALIGN=right COLSPAN=2><FONT SIZE=\"-1\"><B>%lu</B>"               \
      "</FONT></TD></TR>\n",msg_mtot_uu,t_url);
   /* Unique Referrers */
   if (t_ref != 0)
   fprintf(out_fp,"<TR>"                                                     \
      "<TD WIDTH=380><FONT SIZE=\"-1\">%s</FONT></TD>\n"                     \
      "<TD ALIGN=right COLSPAN=2><FONT SIZE=\"-1\"><B>%lu</B>"               \
      "</FONT></TD></TR>\n",msg_mtot_ur,t_ref);
   /* Unique Usernames */
   if (t_user != 0)
   fprintf(out_fp,"<TR>"                                                     \
      "<TD WIDTH=380><FONT SIZE=\"-1\">%s</FONT></TD>\n"                     \
      "<TD ALIGN=right COLSPAN=2><FONT SIZE=\"-1\"><B>%lu</B>"               \
      "</FONT></TD></TR>\n",msg_mtot_ui,t_user);
   /* Unique Agents */
   if (t_agent != 0)
   fprintf(out_fp,"<TR>"                                                     \
      "<TD WIDTH=380><FONT SIZE=\"-1\">%s</FONT></TD>\n"                     \
      "<TD ALIGN=right COLSPAN=2><FONT SIZE=\"-1\"><B>%lu</B>"               \
      "</FONT></TD></TR>\n",msg_mtot_ua,t_agent);
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   /**********************************************/
   /* Hourly/Daily avg/max totals */
   fprintf(out_fp,"<TR>"                                                     \
      "<TH WIDTH=380 BGCOLOR=\"%s\"><FONT SIZE=-1 COLOR=\"%s\">.</FONT></TH>\n"\
      "<TH WIDTH=65 BGCOLOR=\"%s\" ALIGN=right>"                             \
      "<FONT SIZE=-1>%s </FONT></TH>\n"                                      \
      "<TH WIDTH=65 BGCOLOR=\"%s\" ALIGN=right>"                             \
      "<FONT SIZE=-1>%s </FONT></TH></TR>\n",
      GREY,GREY,GREY,msg_h_avg,GREY,msg_h_max);
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   /* Max/Avg Hits per Hour */
   fprintf(out_fp,"<TR>"                                                     \
      "<TD><FONT SIZE=\"-1\">%s</FONT></TD>\n"                               \
      "<TD ALIGN=right WIDTH=65><FONT SIZE=\"-1\"><B>%lu</B></FONT></TD>\n"  \
      "<TD WIDTH=65 ALIGN=right><FONT SIZE=-1><B>%lu</B>"                    \
      "</FONT></TD></TR>\n",msg_mtot_mhh, stream_t_hit/(24*days_in_month),mh_hit);
   /* Max/Avg Hits per Day */
   fprintf(out_fp,"<TR>"                                                     \
      "<TD><FONT SIZE=\"-1\">%s</FONT></TD>\n"                               \
      "<TD ALIGN=right WIDTH=65><FONT SIZE=\"-1\"><B>%lu</B></FONT></TD>\n"  \
      "<TD WIDTH=65 ALIGN=right><FONT SIZE=-1><B>%lu</B>"                    \
      "</FONT></TD></TR>\n",msg_mtot_mhd, stream_t_hit/days_in_month, max_hits);
   /* Max/Avg Visits per Day */
   fprintf(out_fp,"<TR>"                                                     \
      "<TD><FONT SIZE=\"-1\">%s</FONT></TD>\n"                               \
      "<TD ALIGN=right WIDTH=65><FONT SIZE=\"-1\"><B>%lu</B></FONT></TD>\n"  \
      "<TD WIDTH=65 ALIGN=right><FONT SIZE=-1><B>%lu</B>"                    \
      "</FONT></TD></TR>\n",msg_mtot_mvd, stream_t_visit/days_in_month,max_visits);
   /* Max/Avg KBytes per Day */
   fprintf(out_fp,"<TR>"                                                     \
      "<TD><FONT SIZE=\"-1\">%s</FONT></TD>\n"                               \
      "<TD ALIGN=right WIDTH=65><FONT SIZE=\"-1\"><B>%d:%02d:%02d</B></FONT></TD>\n" \
      "<TD WIDTH=65 ALIGN=right><FONT SIZE=-1><B>%d:%02d:%02d</B>"                   \
      "</FONT></TD></TR>\n",msg_mtot_mkd_stream,
      hours(stream_t_xfer/days_in_month),minutes(stream_t_xfer/days_in_month),seconds(stream_t_xfer/days_in_month),
      hours(max_xfer),minutes(max_xfer),seconds(max_xfer));
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   /**********************************************/
   /* response code totals */
   fprintf(out_fp,"<TR><TH COLSPAN=3 ALIGN=center BGCOLOR=\"%s\">\n"         \
           "<FONT SIZE=\"-1\">%s</FONT></TH></TR>\n",GREY,msg_mtot_rc);
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   for (i=0;i<TOTAL_RC;i++)
   {
      if (response[i].count != 0)
         fprintf(out_fp,"<TR><TD><FONT SIZE=\"-1\">%s</FONT></TD>\n"         \
            "<TD ALIGN=right COLSPAN=2><FONT SIZE=\"-1\"><B>%lu</B>"         \
            "</FONT></TD></TR>\n",
            response[i].desc, response[i].count);
   }
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   /**********************************************/

   fprintf(out_fp,"</TABLE>\n");
   fprintf(out_fp,"<P>\n");
}

/*********************************************/
/* DAILY_TOTAL_TABLE - daily totals 4 streams*/
/*********************************************/
void daily_total_table_stream(FILE*out_fp)
{
   int i;

   /* Daily stats */
   fprintf(out_fp,"<TABLE WIDTH=510 BORDER=2 CELLSPACING=1 CELLPADDING=1>\n");
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   /* Daily statistics for ... */
   fprintf(out_fp,"<TR><TH BGCOLOR=\"%s\" COLSPAN=13 ALIGN=center>"          \
           "%s %s %d</TH></TR>\n",
           GREY,msg_dtot_ds_stream,l_month[cur_month-1], cur_year);
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   fprintf(out_fp,"<TR><TH ALIGN=center BGCOLOR=\"%s\">"                     \
                  "<FONT SIZE=\"-1\">%s</FONT></TH>\n"                       \
                  "<TH ALIGN=center BGCOLOR=\"%s\" COLSPAN=2>"               \
                  "<FONT SIZE=\"-1\">%s</FONT></TH>\n"                       \
                  "<TH ALIGN=center BGCOLOR=\"%s\" COLSPAN=2>"               \
                  "<FONT SIZE=\"-1\">%s</FONT></TH>\n"                       \
                  "<TH ALIGN=center BGCOLOR=\"%s\" COLSPAN=2>"               \
                  "<FONT SIZE=\"-1\">%s</FONT></TH>\n"                       \
                  "<TH ALIGN=center BGCOLOR=\"%s\" COLSPAN=2>"               \
                  "<FONT SIZE=\"-1\">%s</FONT></TH>\n"                       \
                  "<TH ALIGN=center BGCOLOR=\"%s\" COLSPAN=2>"               \
                  "<FONT SIZE=\"-1\">%s</FONT></TH></TR>\n",
                  GREY,    msg_h_day,
                  DKGREEN, msg_h_hits,
                  YELLOW,  msg_h_visits,
                  RED,     msg_h_xfer_stream,
		  CYAN,	   msg_d_avgth,
		  ORANGE,  msg_d_avgtv
		);
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");

   /* skip beginning blank days in a month */
   for (i=0;i<hist_lday[cur_month-1];i++) if (tm_hit[i]!=0) break;
   if (i==hist_lday[cur_month-1]) i=0;

   for (;i<hist_lday[cur_month-1];i++)
   {
      fprintf(out_fp,"<TR><TD ALIGN=center>"                                 \
              "<FONT SIZE=\"-1\"><B>%d</B></FONT></TD>\n", i+1);
      fprintf(out_fp,"<TD ALIGN=right>"                                      \
              "<FONT SIZE=\"-1\"><B>%lu</B></FONT></TD>\n"                   \
              "<TD ALIGN=right><FONT SIZE=\"-2\">%3.02f%%</FONT></TD>\n",
              stream_tm_hit[i],PCENT(stream_tm_hit[i],t_hit));
      fprintf(out_fp,"<TD ALIGN=right>"                                      \
              "<FONT SIZE=\"-1\"><B>%lu</B></FONT></TD>\n"                   \
              "<TD ALIGN=right><FONT SIZE=\"-2\">%3.02f%%</FONT></TD>\n",
              stream_tm_visit[i],PCENT(stream_tm_visit[i],stream_t_visit));
      fprintf(out_fp,"<TD ALIGN=right>"                                      \
              "<FONT SIZE=\"-1\"><B>%d:%02d:%02d</B></FONT></TD>\n"                  \
              "<TD ALIGN=right><FONT SIZE=\"-2\">%3.02f%%</FONT></TD>\n",
              hours(stream_tm_xfer[i]),minutes(stream_tm_xfer[i]),seconds(stream_tm_xfer[i]),PCENT(stream_tm_xfer[i],stream_t_xfer));
      fprintf(out_fp,"<TD ALIGN=right>"                                      \
              "<FONT SIZE=\"-1\"><B>%d:%02d:%02d</B></FONT></TD>\n"                  \
              "<TD ALIGN=right><FONT SIZE=\"-2\">%3.02f%%</FONT></TD>\n",
              hours(avg_xfer_hit[i]),minutes(avg_xfer_hit[i]),seconds(avg_xfer_hit[i]),PCENT(avg_xfer_hit[i],stream_t_xfer));
      fprintf(out_fp,"<TD ALIGN=right>"                                      \
              "<FONT SIZE=\"-1\"><B>%d:%02d:%02d</B></FONT></TD>\n"                  \
              "<TD ALIGN=right><FONT SIZE=\"-2\">%3.02f%%</FONT></TD></TR>\n",
              hours(avg_xfer_visit[i]),minutes(avg_xfer_visit[i]),seconds(avg_xfer_visit[i]),PCENT(avg_xfer_visit[i],stream_t_xfer));
   }
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   fprintf(out_fp,"</TABLE>\n");
   fprintf(out_fp,"<P>\n");
}

/*********************************************/
/* STREAM_STATS_TABLE - monthly stats stream table  */
/*********************************************/

void stream_stat_table(FILE*out_fp,int *real_data,int real_size,int overlap)
{
   int maxval=0;
   int maxdur = 0;
   int i,days_in_month;
   int sum = 0;
   int *data;
   int size;

   if (overlap<2) 
   {
     data = real_data;
     size = real_size;
   }
   else
   {
      size = real_size/overlap;
      data = (int*)calloc(size,sizeof(int));
      if (data==NULL) return;
      for( i = 0 ; i<size ; i++ ) data[i] = 0;
      for( i = 0 ; i<real_size ; i++ ) data[i/overlap] += real_data[i];
   }

   days_in_month=(l_day-f_day)+1;

   /* Calculate the max value */
   for( i = 0 ; i<size ; i++ ) 
   { 
      sum += data[i];
      maxval = (data[i]>maxval) ? data[i] : maxval; 
   }
   for( maxdur = size ; (maxdur>0) && (data[maxdur-1]==0) ; maxdur-- );

   /* Stream stats */
   fprintf(out_fp,"<TABLE WIDTH=510 BORDER=2 CELLSPACING=1 CELLPADDING=1>\n");
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   fprintf(out_fp,"<TR><TH BGCOLOR=\"%s\" COLSPAN=13 ALIGN=center>"\
           "%s %s %d</TH></TR>\n",
           GREY,msg_list_dur_ttl,l_month[cur_month-1], cur_year);
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   fprintf(out_fp,"<TR><TH ALIGN=center ROWSPAN=2 BGCOLOR=\"%s\">" \
                  "<FONT SIZE=\"-1\">%s</FONT></TH>\n"             \
                  "<TH ALIGN=center BGCOLOR=\"%s\" COLSPAN=2>"     \
                  "<FONT SIZE=\"-1\">%s</FONT></TH></TR>\n",
                  GREY,    msg_duration,
                  DKGREEN, msg_nb_list);
   fprintf(out_fp,"<TR><TH ALIGN=center BGCOLOR=\"%s\">"           \
                  "<FONT SIZE=\"-2\">%s</FONT></TH>\n"             \
                  "<TH ALIGN=center BGCOLOR=\"%s\" COLSPAN=2>"     \
                  "<FONT SIZE=\"-2\">%s</FONT></TH></TR>\n",
                  DKGREEN, msg_h_total, DKGREEN, msg_h_total);

   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   
   for ( i = 0 ; i<size ; i++ )
   {
      if (data[i]==0) continue;
      fprintf(out_fp,"<TR><TD ALIGN=center><FONT SIZE=\"-1\"><B>%d:%02d -&gt; %d:%02d</B></FONT></TD>\n",(i*overlap)/60,(i*overlap)%60,(i*overlap+overlap)/60,(i*overlap+overlap)%60);
      fprintf(out_fp,
         "<TD ALIGN=right><FONT SIZE=\"-1\"><B>%d</B></FONT></TD>\n" \
         "<TD ALIGN=right><FONT SIZE=\"-2\">%3.02f%%</FONT></TD></TR>\n",
         data[i],PCENT(data[i],sum));
   }
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   fprintf(out_fp,"</TABLE>\n<P>\n");
}


/*********************************************/
/* HOURLY_TOTAL_TABLE - hourly table 4 stream*/
/*********************************************/

void hourly_total_table_stream(FILE*out_fp)
{
   int i,days_in_month;
   double avg_xfer=0.0;
   int t_cxi=0,t_cxo = 0;

   days_in_month=(l_day-f_day)+1;

   /* Hourly stats */
   fprintf(out_fp,"<TABLE WIDTH=510 BORDER=2 CELLSPACING=1 CELLPADDING=1>\n");
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   fprintf(out_fp,"<TR><TH BGCOLOR=\"%s\" COLSPAN=13 ALIGN=center>"\
           "%s %s %d</TH></TR>\n",
           GREY,msg_htot_hs,l_month[cur_month-1], cur_year);
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   fprintf(out_fp,"<TR><TH ALIGN=center ROWSPAN=2 BGCOLOR=\"%s\">" \
                  "<FONT SIZE=\"-1\">%s</FONT></TH>\n"             \
                  "<TH ALIGN=center BGCOLOR=\"%s\" COLSPAN=3>"     \
                  "<FONT SIZE=\"-1\">%s</FONT></TH>\n"             \
                  "<TH ALIGN=center BGCOLOR=\"%s\" COLSPAN=3>"     \
                  "<FONT SIZE=\"-1\">%s</FONT></TH>\n"             \
                  "<TH ALIGN=center BGCOLOR=\"%s\" COLSPAN=3>"     \
                  "<FONT SIZE=\"-1\">%s</FONT></TH>\n"             \
                  "<TH ALIGN=center BGCOLOR=\"%s\" COLSPAN=3>"     \
                  "<FONT SIZE=\"-1\">%s</FONT></TH></TR>\n",
                  GREY,    msg_h_hour,
                  DKGREEN, msg_h_hits,
                  CYAN,    msg_h_cxi,
                  RED,    msg_h_cxo,
                  RED,     msg_h_xfer_stream);
   fprintf(out_fp,"<TR><TH ALIGN=center BGCOLOR=\"%s\">"           \
                  "<FONT SIZE=\"-2\">%s</FONT></TH>\n"             \
                  "<TH ALIGN=center BGCOLOR=\"%s\" COLSPAN=2>"     \
                  "<FONT SIZE=\"-2\">%s</FONT></TH>\n",
                  DKGREEN, msg_h_avg, DKGREEN, msg_h_total);
   fprintf(out_fp,"<TH ALIGN=center BGCOLOR=\"%s\">"               \
                  "<FONT SIZE=\"-2\">%s</FONT></TH>\n"             \
                  "<TH ALIGN=center BGCOLOR=\"%s\" COLSPAN=2>"     \
                  "<FONT SIZE=\"-2\">%s</FONT></TH>\n",
                  CYAN, msg_h_avg, CYAN, msg_h_total);
   fprintf(out_fp,"<TH ALIGN=center BGCOLOR=\"%s\">"               \
                  "<FONT SIZE=\"-2\">%s</FONT></TH>\n"             \
                  "<TH ALIGN=center BGCOLOR=\"%s\" COLSPAN=2>"     \
                  "<FONT SIZE=\"-2\">%s</FONT></TH>\n",
                  RED, msg_h_avg, RED, msg_h_total);
   fprintf(out_fp,"<TH ALIGN=center BGCOLOR=\"%s\">"               \
                  "<FONT SIZE=\"-2\">%s</FONT></TH>\n"             \
                  "<TH ALIGN=center BGCOLOR=\"%s\" COLSPAN=2>"     \
                  "<FONT SIZE=\"-2\">%s</FONT></TH></TR>\n",
                  RED, msg_h_avg, RED, msg_h_total);

   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   for (i=0;i<24;i++)
   {
	t_cxi += stream_th_cxi[i];
	t_cxo += stream_th_cxo[i];
   }
   for (i=0;i<24;i++)
   {
      fprintf(out_fp,"<TR><TD ALIGN=center>"                          \
         "<FONT SIZE=\"-1\"><B>%d</B></FONT></TD>\n",i);
      fprintf(out_fp,
         "<TD ALIGN=right><FONT SIZE=\"-1\"><B>%lu</B></FONT></TD>\n" \
         "<TD ALIGN=right><FONT SIZE=\"-1\"><B>%lu</B></FONT></TD>\n" \
         "<TD ALIGN=right><FONT SIZE=\"-2\">%3.02f%%</FONT></TD>\n",
         stream_th_hit[i]/days_in_month,stream_th_hit[i],
         PCENT(stream_th_hit[i],t_hit));
      fprintf(out_fp,
         "<TD ALIGN=right><FONT SIZE=\"-1\"><B>%lu</B></FONT></TD>\n" \
         "<TD ALIGN=right><FONT SIZE=\"-1\"><B>%lu</B></FONT></TD>\n" \
         "<TD ALIGN=right><FONT SIZE=\"-2\">%3.02f%%</FONT></TD>\n",
         stream_th_cxi[i]/days_in_month,stream_th_cxi[i],
         PCENT(stream_th_cxi[i],t_cxi));
      fprintf(out_fp,
         "<TD ALIGN=right><FONT SIZE=\"-1\"><B>%lu</B></FONT></TD>\n" \
         "<TD ALIGN=right><FONT SIZE=\"-1\"><B>%lu</B></FONT></TD>\n" \
         "<TD ALIGN=right><FONT SIZE=\"-2\">%3.02f%%</FONT></TD>\n",
         stream_th_cxo[i]/days_in_month,stream_th_cxo[i],
         PCENT(stream_th_cxo[i],t_cxo));
      fprintf(out_fp,
         "<TD ALIGN=right><FONT SIZE=\"-1\"><B>%d:%02d:%02d</B></FONT></TD>\n" \
         "<TD ALIGN=right><FONT SIZE=\"-1\"><B>%d:%02d:%02d</B></FONT></TD>\n" \
         "<TD ALIGN=right><FONT SIZE=\"-2\">%3.02f%%</FONT></TD></TR>\n",
	 hours(stream_th_xfer[i]/days_in_month),minutes(stream_th_xfer[i]/days_in_month),seconds(stream_th_xfer[i]/days_in_month),
         hours(stream_th_xfer[i]),minutes(stream_th_xfer[i]),seconds(stream_th_xfer[i]),
         PCENT(stream_th_xfer[i],stream_t_xfer));
      /*avg_file += th_file[i]/days_in_month;*/
      avg_xfer+= (stream_th_xfer[i]/days_in_month)/1024;
   }
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   fprintf(out_fp,"</TABLE>\n<P>\n");
}


/*********************************************/
/* WRITE_MAIN_INDEX - main index.html file   */
/*********************************************/

int write_main_index_stream()
{
   /* create main index file */

   int  i,days_in_month;
   int  lyear=0;
   int	s_mth=0;
   int  mc=0;
   double  gt_hit=0.0;
   double  gt_xfer=0.0;
   double  gt_visits=0.0;
   char    index_fname[256];
   char    buffer[BUFSIZE];
   u_long  hist_empty[12];
   u_long  gt_avth=0;
   u_long  gt_avtv=0;

   FILE*out_fp = NULL;

   for(i=0 ; i<12 ; i++ ) hist_empty[i]=0;

   if (verbose>1) printf("%s\n",msg_gen_sum);

   sprintf(buffer,"%s %s",msg_main_us,hname);

   for (i=0;i<12;i++)                   /* get last month in history */
   {
      if (hist_year[i]>lyear)
       { lyear=hist_year[i]; s_mth=hist_month[i]; }
      if (hist_year[i]==lyear)
      {
         if (hist_month[i]>=s_mth)
            s_mth=hist_month[i];
      }
   }

   i=(s_mth==12)?1:s_mth+1;

   year_graph6x_msg("usage.png",        /* filename          */
                   buffer,              /* graph title       */
                   i,                   /* last month        */
                   hist_hit_stream,msg_h_hits,COLOR1,            /* data set 1        */
                   hist_visit_stream,msg_h_visits,yellow,         	  /* data set 2        */
                   hist_avth,msg_d_avgth,COLOR3,          /* data set 3        */
                   hist_xfer_stream,msg_h_xfer_stream,COLOR4,    /* data set 4        */
                   hist_empty,"",COLOR5,          	  /* data set 5        */
                   hist_avtv,msg_d_avgtv,COLOR6);         /* data set 6        */

   /* now do html stuff... */
   sprintf(index_fname,"index.%s",html_ext);

   if ( (out_fp=fopen(index_fname,"w")) == NULL)
   {
      if (verbose)
      fprintf(stderr,"%s %s!\n",msg_no_open,index_fname);
      return 1;
   }
   write_html_head(msg_main_per, out_fp);
   /* year graph */
   fprintf(out_fp,"<IMG SRC=\"usage.png\" ALT=\"%s\" "    \
                  "HEIGHT=256 WIDTH=512><P>\n",buffer);
   /* month table */
   fprintf(out_fp,"<TABLE WIDTH=600 BORDER=2 CELLSPACING=1 CELLPADDING=1>\n");
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   fprintf(out_fp,"<TR><TH COLSPAN=11 BGCOLOR=\"%s\" ALIGN=center>",GREY);
   fprintf(out_fp,"%s</TH></TR>\n",msg_main_sum);
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   fprintf(out_fp,"<TR><TH ALIGN=left ROWSPAN=2 BGCOLOR=\"%s\"><FONT SIZE=\"-1\">%s</FONT></TH>\n",GREY,msg_h_mth);
   fprintf(out_fp,"<TH ALIGN=center COLSPAN=2 BGCOLOR=\"%s\"><FONT SIZE=\"-1\">%s</FONT></TH>\n",GREY,msg_main_da);
   fprintf(out_fp,"<TH ALIGN=center COLSPAN=6 BGCOLOR=\"%s\"><FONT SIZE=\"-1\">%s</FONT></TH></TR>\n",GREY,msg_main_mt);
   /*Average*/
   fprintf(out_fp,"<TH ALIGN=center BGCOLOR=\"%s\"><FONT SIZE=\"-1\">%s</FONT></TH>\n",DKGREEN,msg_h_hits);
   fprintf(out_fp,"<TH ALIGN=center BGCOLOR=\"%s\"><FONT SIZE=\"-1\">%s</FONT></TH>\n",YELLOW,msg_h_visits);
   /*Totals*/
   fprintf(out_fp,"<TH ALIGN=center BGCOLOR=\"%s\"><FONT SIZE=\"-1\">%s</FONT></TH>\n",RED,msg_h_xfer_stream);
   fprintf(out_fp,"<TH ALIGN=center BGCOLOR=\"%s\"><FONT SIZE=\"-1\">%s</FONT></TH>\n",DKGREEN,msg_h_hits);
   fprintf(out_fp,"<TH ALIGN=center BGCOLOR=\"%s\"><FONT SIZE=\"-1\">%s</FONT></TH>\n",YELLOW,msg_h_visits);
   fprintf(out_fp,"<TH ALIGN=center BGCOLOR=\"%s\"><FONT SIZE=\"-1\">%s</FONT></TH>\n",ORANGE,msg_d_avgth);
   fprintf(out_fp,"<TH ALIGN=center BGCOLOR=\"%s\"><FONT SIZE=\"-1\">%s</FONT></TH></TR>\n",YELLOW,msg_d_avgtv);
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   for (i=0;i<12;i++)
   {
      if (--s_mth < 0) s_mth = 11;
      if ((hist_month[s_mth]==0) && (hist_files[s_mth]==0)) continue;
      days_in_month=(hist_lday[s_mth]-hist_fday[s_mth])+1;
      fprintf(out_fp,"<TR><TD NOWRAP><A HREF=\"usage_%04d%02d.%s\">"      \
                     "<FONT SIZE=\"-1\">%s %d</FONT></A></TD>\n",
                      hist_year[s_mth], hist_month[s_mth], html_ext,
                      s_month[hist_month[s_mth]-1], hist_year[s_mth]);
      fprintf(out_fp,"<TD ALIGN=right><FONT SIZE=\"-1\">%lu</FONT></TD>\n",
                      hist_hit_stream[s_mth]/days_in_month);
      fprintf(out_fp,"<TD ALIGN=right><FONT SIZE=\"-1\">%lu</FONT></TD>\n",
                      hist_visit_stream[s_mth]/days_in_month);
      /*Totals*/
      fprintf(out_fp,"<TD ALIGN=right><FONT SIZE=\"-1\">%d:%02d:%02d</FONT></TD>\n",
                      hours(hist_xfer_stream[s_mth]*1024),minutes(hist_xfer_stream[s_mth]*1024),seconds(hist_xfer_stream[s_mth]*1024));
      fprintf(out_fp,"<TD ALIGN=right><FONT SIZE=\"-1\">%lu</FONT></TD>\n",
                      hist_hit_stream[s_mth]);
      fprintf(out_fp,"<TD ALIGN=right><FONT SIZE=\"-1\">%lu</FONT></TD>\n",
                      hist_visit_stream[s_mth]);
      fprintf(out_fp,"<TD ALIGN=right><FONT SIZE=\"-1\">%d:%02d:%02d</FONT></TD>\n",
                      hours(hist_avth[s_mth]*1024),minutes(hist_avth[s_mth]*1024),seconds(hist_avth[s_mth]*1024));
      fprintf(out_fp,"<TD ALIGN=right><FONT SIZE=\"-1\">%d:%02d:%02d</FONT></TD></TR>\n",
                      hours(hist_avtv[s_mth]*1024),minutes(hist_avtv[s_mth]*1024),seconds(hist_avtv[s_mth]*1024));
      gt_hit   += hist_hit_stream[s_mth];
      gt_xfer  += hist_xfer_stream[s_mth]*1024;
      gt_visits+= hist_visit_stream[s_mth];
      gt_avth  += hist_avth[s_mth]*1024;
      gt_avtv  += hist_avtv[s_mth]*1024;
      mc++; /* Month count */
   }
   if (mc!=0)
   {
	gt_avth /= mc;
	gt_avtv /= mc;
   }

   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   fprintf(out_fp,"<TR><TH BGCOLOR=\"%s\" COLSPAN=3 ALIGN=left><FONT SIZE=\"-1\">%s</FONT></TH>\n",GREY,msg_h_totals);
   fprintf(out_fp,"<TH BGCOLOR=\"%s\" ALIGN=right><FONT SIZE=\"-1\">%d:%02d:%02d</FONT></TH>\n",GREY,hours(gt_xfer),minutes(gt_xfer),seconds(gt_xfer));
   fprintf(out_fp,"<TH BGCOLOR=\"%s\" ALIGN=right><FONT SIZE=\"-1\">%.0f</FONT></TH>\n",GREY,gt_hit);
   fprintf(out_fp,"<TH BGCOLOR=\"%s\" ALIGN=right><FONT SIZE=\"-1\">%.0f</FONT></TH>\n",GREY,gt_visits);
   fprintf(out_fp,"<TH BGCOLOR=\"%s\" ALIGN=right><FONT SIZE=\"-1\">%d:%02d:%02d</FONT></TH>\n",GREY,hours(gt_avth),minutes(gt_avth),seconds(gt_avth));
   fprintf(out_fp,"<TH BGCOLOR=\"%s\" ALIGN=right><FONT SIZE=\"-1\">%d:%02d:%02d</FONT></TH>\n",GREY,hours(gt_avtv),minutes(gt_avtv),seconds(gt_avtv));
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   fprintf(out_fp,"</TABLE>\n");
   write_html_tail(out_fp);
   fclose(out_fp);
   return 0;
}


/*********************************************/
/* TOP_URLS_TABLE - generate top n table     */
/*********************************************/

void top_urls_table_stream(FILE*out_fp,int flag)
{
   u_long cnt=0,u_reg=0,u_grp=0,u_hid=0, tot_num;
   int i;
   UNODEPTR uptr, *pointer;
   unsigned char *cpstream;              /* char pointer for stream analysis */

   cnt=a_ctr; pointer=u_array;
   while (cnt--)
   {
      /* calculate totals */
      switch ( (int)((UNODEPTR)(*pointer)->flag) )
      {
         case OBJ_REG:  u_reg++;  break;
         case OBJ_GRP:  u_grp++;  break;
         case OBJ_HIDE: u_hid++;  break;
      }
      pointer++;
   }

   if ( (tot_num=u_reg+u_grp)==0 ) return;              /* split if none    */
   i=(flag)?ntop_urlsK:ntop_urls;                       /* Hits or KBytes?? */
   if (tot_num > i) tot_num = i;                        /* get max to do... */
   if ((!flag) || (flag&&!ntop_urls))                   /* now do <A> tag   */
      fprintf(out_fp,"<A NAME=\"TOPURLS\"></A>\n");

   fprintf(out_fp,"<TABLE WIDTH=510 BORDER=2 CELLSPACING=1 CELLPADDING=1>\n");
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   if (flag) fprintf(out_fp,"<TR><TH BGCOLOR=\"%s\" ALIGN=CENTER COLSPAN=6>"  \
           "%s %lu %s %lu %s %s %s</TH></TR>\n",
           GREY,msg_top_top,tot_num,msg_top_of,
           t_url,msg_top_u,msg_h_by,msg_h_xfer_stream);
   else fprintf(out_fp,"<TR><TH BGCOLOR=\"%s\" ALIGN=CENTER COLSPAN=6>"   \
           "%s %lu %s %lu %s</TH></TR>\n",
           GREY,msg_top_top,tot_num,msg_top_of,t_url,msg_top_u);
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   fprintf(out_fp,"<TR><TH BGCOLOR=\"%s\" ALIGN=center>"                  \
                  "<FONT SIZE=\"-1\">#</FONT></TH>\n",GREY);
   fprintf(out_fp,"<TH BGCOLOR=\"%s\" ALIGN=center COLSPAN=2>"            \
                  "<FONT SIZE=\"-1\">%s</FONT></TH>\n",
                  DKGREEN,msg_h_cxi);
   fprintf(out_fp,"<TH BGCOLOR=\"%s\" ALIGN=center COLSPAN=2>"            \
                  "<FONT SIZE=\"-1\">%s</FONT></TH>\n",
                  RED,msg_h_xfer_stream);
   fprintf(out_fp,"<TH BGCOLOR=\"%s\" ALIGN=center>"                      \
                  "<FONT SIZE=\"-1\">%s</FONT></TH></TR>\n",
                  CYAN,msg_h_url);
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");

   pointer=u_array; i=0;
   while (tot_num)
   {
      uptr=*pointer++;             /* point to the URL node */
      if (uptr->flag != OBJ_HIDE)
      {
	if ((cpstream=isinglist(stream_list,uptr->string))!=NULL)
        {
	 /* cpstream is the average bit rate */
         /*int bitrate  = atoi(cpstream);*/
	 /* uptr->xfer is already in seconds*1024 */
         /*int duration = (((u_long)uptr->xfer)<<3)/bitrate*1024;*/ /* Download duration in seconds */
	 double duration = uptr->xfer; /* Duration in seconds*1024 */

         /* shade grouping? */
         if (shade_groups && (uptr->flag==OBJ_GRP))
            fprintf(out_fp,"<TR BGCOLOR=\"%s\">\n", GRPCOLOR);
         else fprintf(out_fp,"<TR>\n");

         fprintf(out_fp,
            "<TD ALIGN=center><FONT SIZE=\"-1\"><B>%d</B></FONT></TD>\n" \
            "<TD ALIGN=right><FONT SIZE=\"-1\"><B>%lu</B></FONT></TD>\n" \
            "<TD ALIGN=right><FONT SIZE=\"-2\">%3.02f%%</FONT></TD>\n"   \
            "<TD ALIGN=right><FONT SIZE=\"-1\"><B>%d:%02d:%02d</B></FONT></TD>\n"\
            "<TD ALIGN=right><FONT SIZE=\"-2\">%3.02f%%</FONT></TD>\n"   \
            "<TD ALIGN=left NOWRAP><FONT SIZE=\"-1\">",
            i+1,uptr->count,
            (t_hit==0)?0:((double)uptr->count/stream_t_hit)*100.0,
            hours(duration),minutes(duration),seconds(duration),
            (t_xfer==0)?0:(duration/stream_t_xfer)*100.0);

         if (uptr->flag==OBJ_GRP)
         {
            if (hlite_groups)
               fprintf(out_fp,"<STRONG>%s</STRONG></FONT></TD></TR>\n",
                uptr->string);
            else fprintf(out_fp,"%s</FONT></TD></TR>\n",uptr->string);
         }
         else 
	 {
            /* check for a service prefix (ie: http://) */
            if (strstr(uptr->string,"://")!=NULL)
               fprintf(out_fp,"<A HREF=\"%s\">%s</A></FONT></TD></TR>\n",
                 uptr->string,uptr->string);
	    else
            {
               if (log_type == LOG_FTP) /* FTP log? */
                   fprintf(out_fp,"%s</FONT></TD></TR>\n",uptr->string);
               else
               {             /* Web log  */
                  if (use_https)
                     /* secure server mode, use https:// */
                     fprintf(out_fp,
                     "<A HREF=\"https://%s%s\">%s</A></FONT></TD></TR>\n",
                      hname,uptr->string,uptr->string);
                   else
                      /* otherwise use standard 'http://' */
                      fprintf(out_fp,
                      "<A HREF=\"http://%s%s\">%s</A></FONT></TD></TR>\n",
                      hname,uptr->string,uptr->string);
               }
            }
	 }
         i++;
        }
        tot_num--;
      }
   }
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   if ((!flag) || (flag&&!ntop_urls))
   {
      if ( (all_urls) && ((u_reg+u_grp)>ntop_urls) )
      {
         if (all_urls_page_stream(u_reg, u_grp))
         {
            fprintf(out_fp,"<TR BGCOLOR=\"%s\">",GRPCOLOR);
            fprintf(out_fp,"<TD COLSPAN=6 ALIGN=\"center\">\n");
            fprintf(out_fp,"<FONT SIZE=\"-1\">");
            fprintf(out_fp,"<A HREF=\"./url_%04d%02d.%s\">",
                    cur_year,cur_month,html_ext);
            fprintf(out_fp,"%s</A></TD></TR>\n",msg_v_urls);
            if (flag)   /* do we need to sort first? */
               qsort(u_array,a_ctr,sizeof(UNODEPTR),qs_url_cmph);
         }
      }
   }
   fprintf(out_fp,"</TABLE>\n<P>\n");
}



/*********************************************/
/* ALL_URLS_PAGE - HTML page of all urls     */
/*********************************************/

int all_urls_page_stream(u_long u_reg, u_long u_grp)
{
   UNODEPTR uptr, *pointer;
   char     url_fname[256], buffer[256];
   FILE     *out_fp;
   int      i=(u_grp)?1:0;
   unsigned char *cpstream;              /* char pointer for stream analysis */

   /* generate file name */
   sprintf(url_fname,"url_%04d%02d.%s",cur_year,cur_month,html_ext);

   /* open file */
   if ( (out_fp=open_out_file(url_fname))==NULL ) return 0;

   sprintf(buffer,"%s %d - %s",l_month[cur_month-1],cur_year,msg_h_url);
   write_html_head(buffer, out_fp);

   fprintf(out_fp,"<FONT SIZE=\"-1\"></CENTER><PRE>\n");

   fprintf(out_fp," %12s      %12s      %s\n",
           msg_h_hits,msg_h_xfer_stream,msg_h_url);
   fprintf(out_fp,"----------------  ----------------  " \
                  "--------------------\n\n");

   /* do groups first (if any) */
   pointer=u_array;
   while (u_grp)
   {
      uptr=*pointer++;
      if (uptr->flag == OBJ_GRP)
      {
	if ((cpstream=isinglist(stream_list,uptr->string))!=NULL)
        {
	 /* cpstream is the average bit rate */
         int bitrate  = atoi(cpstream);
         int duration = (((u_long)uptr->xfer)<<3)/bitrate*1024; /* Download duration in seconds */
         fprintf(out_fp,"%-8lu %6.02f%%  %d:%02d:%02d %6.02f%%  %s\n",
            uptr->count,
            (t_hit==0)?0:((double)uptr->count/t_hit)*100.0,
	    hours(duration),minutes(duration),seconds(duration),
            (stream_t_xfer==0)?0:((double)uptr->xfer/stream_t_xfer)*100.0,
            uptr->string);
	}
	else
	{
         fprintf(out_fp,"%-8lu %6.02f%%  %8.0f %6.02f%%  %s\n",
            uptr->count,
            (t_hit==0)?0:((double)uptr->count/t_hit)*100.0,
            uptr->xfer/1024,
            (t_xfer==0)?0:((double)uptr->xfer/t_xfer)*100.0,
            uptr->string);
	}
         u_grp--;
      }
   }

   if (i) fprintf(out_fp,"\n");

   /* now do invididual sites (if any) */
   pointer=u_array;
   while (u_reg)
   {
      uptr=*pointer++;
      if (uptr->flag == OBJ_REG)
      {
         fprintf(out_fp,"%-8lu %6.02f%%  %d:%02d:%02d %6.02f%%  %s\n",
            uptr->count,
            (t_hit==0)?0:((double)uptr->count/t_hit)*100.0,
	    hours(uptr->xfer),minutes(uptr->xfer),seconds(uptr->xfer),
            (stream_t_xfer==0)?0:((double)uptr->xfer/stream_t_xfer)*100.0,
            uptr->string);
         u_reg--;
      }
   }

   fprintf(out_fp,"</PRE></FONT>\n");
   write_html_tail(out_fp);
   fclose(out_fp);
   return 1;
}



/*********************************************/
/* DUMP_ALL_URLS - dump all urls to tab file */
/*********************************************/

void dump_all_urls_stream()
{
   UNODEPTR uptr, *pointer;
   FILE     *out_fp;
   char     filename[256];
   u_long   cnt=a_ctr;
   unsigned char *cpstream;              /* char pointer for stream analysis */

   /* generate file name */
   sprintf(filename,"%s/url_%04d%02d.%s",
      (dump_path)?dump_path:".",cur_year,cur_month,dump_ext);

   /* open file */
   if ( (out_fp=open_out_file(filename))==NULL ) return;

   /* need a header? */
   if (dump_header)
   {
      fprintf(out_fp,"%s\t%s\t%s\n",msg_h_hits,msg_h_xfer_stream,msg_h_url);
   }

   /* dump 'em */
   pointer=u_array;
   while (cnt)
   {
      uptr=*pointer++;
      if (uptr->flag != OBJ_GRP)
      {
	if ((cpstream=isinglist(stream_list,uptr->string))!=NULL)
        {
	 /* cpstream is the average bit rate */
         int bitrate  = atoi(cpstream);
         int duration = (((u_long)uptr->xfer)<<3)/bitrate*1024; /* Download duration in seconds */
         fprintf(out_fp,"%lu\t%d:%02d:%02d\t%s\n",
            uptr->count,hours(duration),minutes(duration),seconds(duration),uptr->string);
	}
	else
	{
         fprintf(out_fp,"%lu\t%.0f\t%s\n",
            uptr->count,uptr->xfer/1024,uptr->string);
	}
      }
      cnt--;
   }
   fclose(out_fp);
   return;
}


/*********************************************/
/* TOP_SITES_TABLE - generate top n table    */
/*********************************************/

void top_sites_table_stream(FILE*out_fp,int flag)
{
   u_long cnt=0, h_reg=0, h_grp=0, h_hid=0, tot_num;
   int i;
   HNODEPTR hptr, *pointer;

   cnt=a_ctr; pointer=h_array;
   while(cnt--)
   {
      /* calculate totals */
      switch ( (int)((HNODEPTR)(*pointer)->flag) )
      {
         case OBJ_REG:   h_reg++;  break;
         case OBJ_GRP:   h_grp++;  break;
         case OBJ_HIDE:  h_hid++;  break;
      }
      pointer++;
   }

   if ( (tot_num=h_reg+h_grp)==0 ) return;              /* split if none    */
   i=(flag)?ntop_sitesK:ntop_sites;                     /* Hits or KBytes?? */
   if (tot_num > i) tot_num = i;                        /* get max to do... */

   if ((!flag) || (flag&&!ntop_sites))                  /* now do <A> tag   */
      fprintf(out_fp,"<A NAME=\"TOPSITES\"></A>\n");

   fprintf(out_fp,"<TABLE WIDTH=510 BORDER=2 CELLSPACING=1 CELLPADDING=1>\n");
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   if (flag) fprintf(out_fp,"<TR><TH BGCOLOR=\"%s\" ALIGN=CENTER COLSPAN=8>" \
           "%s %lu %s %lu %s %s %s</TH></TR>\n",
           GREY, msg_top_top,tot_num,msg_top_of,
           t_site,msg_top_s,msg_h_by,msg_h_xfer_stream);
   else      fprintf(out_fp,"<TR><TH BGCOLOR=\"%s\" ALIGN=CENTER COLSPAN=8>" \
           "%s %lu %s %lu %s</TH></TR>\n",
           GREY,msg_top_top, tot_num, msg_top_of, t_site, msg_top_s);
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   fprintf(out_fp,"<TR><TH BGCOLOR=\"%s\" ALIGN=center>"                    \
          "<FONT SIZE=\"-1\">#</FONT></TH>\n",GREY);
   fprintf(out_fp,"<TH BGCOLOR=\"%s\" ALIGN=center COLSPAN=2>"              \
          "<FONT SIZE=\"-1\">%s</FONT></TH>\n",DKGREEN,msg_h_hits);
   fprintf(out_fp,"<TH BGCOLOR=\"%s\" ALIGN=center COLSPAN=2>"              \
          "<FONT SIZE=\"-1\">%s</FONT></TH>\n",RED,msg_h_xfer_stream);
   fprintf(out_fp,"<TH BGCOLOR=\"%s\" ALIGN=center COLSPAN=2>"              \
          "<FONT SIZE=\"-1\">%s</FONT></TH>\n",YELLOW,msg_h_visits);
   fprintf(out_fp,"<TH BGCOLOR=\"%s\" ALIGN=center>"                        \
          "<FONT SIZE=\"-1\">%s</FONT></TH></TR>\n",CYAN,msg_h_hname);
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");

   pointer=h_array; i=0;
   while(tot_num)
   {
      hptr=*pointer++;
      if (hptr->flag != OBJ_HIDE)
      {
         /* shade grouping? */
         if (shade_groups && (hptr->flag==OBJ_GRP))
            fprintf(out_fp,"<TR BGCOLOR=\"%s\">\n", GRPCOLOR);
         else fprintf(out_fp,"<TR>\n");

         fprintf(out_fp,
              "<TD ALIGN=center><FONT SIZE=\"-1\"><B>%d</B></FONT></TD>\n"  \
              "<TD ALIGN=right><FONT SIZE=\"-1\"><B>%lu</B></FONT></TD>\n"  \
              "<TD ALIGN=right><FONT SIZE=\"-2\">%3.02f%%</FONT></TD>\n"    \
              "<TD ALIGN=right><FONT SIZE=\"-1\"><B>%d:%02d:%02d</B></FONT></TD>\n" \
              "<TD ALIGN=right><FONT SIZE=\"-2\">%3.02f%%</FONT></TD>\n"    \
              "<TD ALIGN=right><FONT SIZE=\"-1\"><B>%lu</B></FONT></TD>\n"  \
              "<TD ALIGN=right><FONT SIZE=\"-2\">%3.02f%%</FONT></TD>\n"    \
              "<TD ALIGN=left NOWRAP><FONT SIZE=\"-1\">",
              i+1,hptr->count,
              (t_hit==0)?0:((double)hptr->count/t_hit)*100.0,
	      hours(hptr->xfer),minutes(hptr->xfer),seconds(hptr->xfer),
              (stream_t_xfer==0)?0:((double)hptr->xfer/stream_t_xfer)*100.0,
	      hptr->visit,
              (t_visit==0)?0:((double)hptr->visit/t_visit)*100.0);

         if ((hptr->flag==OBJ_GRP)&&hlite_groups)
             fprintf(out_fp,"<STRONG>%s</STRONG></FONT></TD></TR>\n",
               hptr->string);
         else fprintf(out_fp,"%s</FONT></TD></TR>\n",
               hptr->string);
         tot_num--;
         i++;
      }
   }

   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   if ((!flag) || (flag&&!ntop_sites))
   {
      if ( (all_sites) && ((h_reg+h_grp)>ntop_sites) )
      {
         if (all_sites_page(h_reg, h_grp))
         {
            fprintf(out_fp,"<TR BGCOLOR=\"%s\">",GRPCOLOR);
            fprintf(out_fp,"<TD COLSPAN=10 ALIGN=\"center\">\n");
            fprintf(out_fp,"<FONT SIZE=\"-1\">");
            fprintf(out_fp,"<A HREF=\"./site_%04d%02d.%s\">",
                    cur_year,cur_month,html_ext);
            fprintf(out_fp,"%s</A></TD></TR>\n",msg_v_sites);
            if (flag)   /* do we need to sort? */
               qsort(h_array,a_ctr,sizeof(HNODEPTR),qs_site_cmph);
         }
      }
   }
   fprintf(out_fp,"</TABLE>\n<P>\n");
}

/*********************************************/
/* TOP_CTRY_TABLE - top countries table      */
/*********************************************/

void top_ctry_table_stream(FILE *out_fp)
{
   int i,j,x,tot_num=0,tot_ctry=0;
   int ctry_fnd;
   u_long idx;
   HNODEPTR hptr;
   char *domain;
   u_long pie_data[10];
   char   *pie_legend[10];
   char   pie_title[48];
   char   pie_fname[48];

   extern int ctry_graph;  /* include external flag */

   /* scan hash table adding up domain totals */
   for (i=0;i<MAXHASH;i++)
   {
      hptr=sm_htab[i];
      while (hptr!=NULL)
      {
         if (hptr->flag != OBJ_GRP)   /* ignore group totals */
         {
            domain = hptr->string+strlen(hptr->string)-1;
            while ( (*domain!='.')&&(domain!=hptr->string)) domain--;
            if ((domain==hptr->string)||(isdigit((int)*++domain)))
            {
               ctry[0].count+=hptr->count;
               ctry[0].xfer +=hptr->xfer;
            }
            else
            {
               ctry_fnd=0;
               idx=ctry_idx(domain);
               for (j=0;ctry[j].desc;j++)
               {
                  if (idx==ctry[j].idx)
                  {
                     ctry[j].count+=hptr->count;
                     ctry[j].xfer +=hptr->xfer;
                     ctry_fnd=1;
                     break;
                  }
               }
               if (!ctry_fnd)
               {
                  ctry[0].count+=hptr->count;
                  ctry[0].xfer +=hptr->xfer;
               }
            }
         }
         hptr=hptr->next;
      }
   }

   for (i=0;ctry[i].desc;i++)
   {
      if (ctry[i].count!=0) tot_ctry++;
      for (j=0;j<ntop_ctrys;j++)
      {
         if (top_ctrys[j]==NULL) { top_ctrys[j]=&ctry[i]; break; }
         else
         {
            if (ctry[i].count > top_ctrys[j]->count)
            {
               for (x=ntop_ctrys-1;x>j;x--)
                  top_ctrys[x]=top_ctrys[x-1];
               top_ctrys[x]=&ctry[i];
               break;
            }
         }
      }
   }

   /* put our anchor tag first... */
   fprintf(out_fp,"<A NAME=\"TOPCTRYS\"></A>\n");

   /* generate pie chart if needed */
   if (ctry_graph)
   {
      for (i=0;i<10;i++) pie_data[i]=0;             /* init data array      */
      if (ntop_ctrys<10) j=ntop_ctrys; else j=10;   /* ensure data size     */

      for (i=0;i<j;i++)
      {
         pie_data[i]=top_ctrys[i]->xfer;            /* load the array       */
         pie_legend[i]=top_ctrys[i]->desc;
      }
      sprintf(pie_title,"%s %s %d",msg_ctry_use,l_month[cur_month-1],cur_year);
      sprintf(pie_fname,"ctry_usage_%04d%02d.png",cur_year,cur_month);

      pie_chart(pie_fname,pie_title,stream_t_xfer,pie_data,pie_legend);  /* do it   */

      /* put the image tag in the page */
      fprintf(out_fp,"<IMG SRC=\"%s\" ALT=\"%s\" " \
                  "HEIGHT=300 WIDTH=512><P>\n",pie_fname,pie_title);
   }

   /* Now do the table */
   for (i=0;i<ntop_ctrys;i++) if (top_ctrys[i]->count!=0) tot_num++;
   fprintf(out_fp,"<TABLE WIDTH=510 BORDER=2 CELLSPACING=1 CELLPADDING=1>\n");
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   fprintf(out_fp,"<TR><TH BGCOLOR=\"%s\" ALIGN=CENTER COLSPAN=8>"         \
           "%s %d %s %d %s</TH></TR>\n",
           GREY,msg_top_top,tot_num,msg_top_of,tot_ctry,msg_top_c);
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   fprintf(out_fp,"<TR><TH BGCOLOR=\"%s\" ALIGN=center>"                   \
          "<FONT SIZE=\"-1\">#</FONT></TH>\n",GREY);
   fprintf(out_fp,"<TH BGCOLOR=\"%s\" ALIGN=center COLSPAN=2>"             \
          "<FONT SIZE=\"-1\">%s</FONT></TH>\n",DKGREEN,msg_h_hits);
   fprintf(out_fp,"<TH BGCOLOR=\"%s\" ALIGN=center COLSPAN=2>"             \
          "<FONT SIZE=\"-1\">%s</FONT></TH>\n",RED,msg_h_xfer_stream);
   fprintf(out_fp,"<TH BGCOLOR=\"%s\" ALIGN=center>"                       \
          "<FONT SIZE=\"-1\">%s</FONT></TH></TR>\n",CYAN,msg_h_ctry);
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   for (i=0;i<ntop_ctrys;i++)
   {
      if (top_ctrys[i]->count!=0)
      fprintf(out_fp,"<TR>"                                                \
              "<TD ALIGN=center><FONT SIZE=\"-1\"><B>%d</B></FONT></TD>\n" \
              "<TD ALIGN=right><FONT SIZE=\"-1\"><B>%lu</B></FONT></TD>\n" \
              "<TD ALIGN=right><FONT SIZE=\"-2\">%3.02f%%</FONT></TD>\n"   \
              "<TD ALIGN=right><FONT SIZE=\"-1\"><B>%d:%02d:%02d</B></FONT></TD>\n" \
              "<TD ALIGN=right><FONT SIZE=\"-2\">%3.02f%%</FONT></TD>\n"   \
              "<TD ALIGN=left NOWRAP><FONT SIZE=\"-1\">%s</FONT></TD></TR>\n",
              i+1,top_ctrys[i]->count,
              (t_hit==0)?0:((double)top_ctrys[i]->count/t_hit)*100.0,
              hours(top_ctrys[i]->xfer),minutes(top_ctrys[i]->xfer),seconds(top_ctrys[i]->xfer),
              (stream_t_xfer==0)?0:((double)top_ctrys[i]->xfer/stream_t_xfer)*100.0,
              top_ctrys[i]->desc);
   }
   fprintf(out_fp,"<TR><TH HEIGHT=4></TH></TR>\n");
   fprintf(out_fp,"</TABLE>\n<P>\n");
}
